﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SchoolContracts.BindingModels;
using SchoolContracts.BusinessLogicsContracts;
using SchoolContracts.SearchModels;
using SchoolContracts.StorageContracts;
using SchoolContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace SchoolBusinessLogic.BusinessLogic
{
	public class CoursesForStudyLogic: ICoursesForStudyLogic
	{
		private readonly ILogger _logger;
		private readonly ICoursesForStudyStorage _coursesforstudyStorage;
		public CoursesForStudyLogic(ILogger<CoursesForStudyLogic> logger, ICoursesForStudyStorage coursesforstudyStorage)
		{
			_logger = logger;
			_coursesforstudyStorage = coursesforstudyStorage;
		}
		public List<CoursesForStudyViewModel>? ReadList(CoursesForStudySearchModel? model)
		{
			_logger.LogInformation("ReadList. Id: {Id}", model?.Id);
			var list = model == null ? _coursesforstudyStorage.GetFullList() : _coursesforstudyStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count: {Count}", list.Count);
			return list;
		}
		public CoursesForStudyViewModel? ReadElement(CoursesForStudySearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. Id: {Id}", model.Id);
			var element = _coursesforstudyStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement found. Id: {Id}", element.Id);
			return element;
		}
		public bool Create(CoursesForStudyBindingModel model)
		{
			CheckModel(model);
			if (_coursesforstudyStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(CoursesForStudyBindingModel model)
		{
			CheckModel(model);
			if (_coursesforstudyStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(CoursesForStudyBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_coursesforstudyStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(CoursesForStudyBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			_logger.LogInformation("CoursesForStudy. Id: {Id}", model.Id);
		}
	}
}
