﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SchoolContracts.BindingModels;
using SchoolContracts.BusinessLogicsContracts;
using SchoolContracts.SearchModels;
using SchoolContracts.StorageContracts;
using SchoolContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace SchoolBusinessLogic.BusinessLogic
{
	public class CoursesLogic: ICoursesLogic
	{
		private readonly ILogger _logger;
		private readonly ICoursesStorage _coursesStorage;
		public CoursesLogic(ILogger<CoursesLogic> logger, ICoursesStorage coursesStorage)
		{
			_logger = logger;
			_coursesStorage = coursesStorage;
		}
		public List<CoursesViewModel>? ReadList(CoursesSearchModel? model)
		{
			_logger.LogInformation("ReadList. Id: {Id}", model?.Id);
			var list = model == null ? _coursesStorage.GetFullList() : _coursesStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count: {Count}", list.Count);
			return list;
		}
		public CoursesViewModel? ReadElement(CoursesSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. Id: {Id}", model.Id);
			var element = _coursesStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement found. Id: {Id}", element.Id);
			return element;
		}
		public bool Create(CoursesBindingModel model)
		{
			CheckModel(model);
			if (_coursesStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(CoursesBindingModel model)
		{
			CheckModel(model);
			if (_coursesStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(CoursesBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_coursesStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(CoursesBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			_logger.LogInformation("Courses. Id: {Id}", model.Id);
		}
	}
}
