﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SchoolContracts.BindingModels;
using SchoolContracts.BusinessLogicsContracts;
using SchoolContracts.SearchModels;
using SchoolContracts.StorageContracts;
using SchoolContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace SchoolBusinessLogic.BusinessLogic
{
	public class ItemForStudyLogic: IItemForStudyLogic
	{
		private readonly ILogger _logger;
		private readonly IItemForStudyStorage _itemforstudyStorage;
		public ItemForStudyLogic(ILogger<ItemForStudyLogic> logger, IItemForStudyStorage itemforstudyStorage)
		{
			_logger = logger;
			_itemforstudyStorage = itemforstudyStorage;
		}
		public List<ItemForStudyViewModel>? ReadList(ItemForStudySearchModel? model)
		{
			_logger.LogInformation("ReadList. Id: {Id}", model?.Id);
			var list = model == null ? _itemforstudyStorage.GetFullList() : _itemforstudyStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count: {Count}", list.Count);
			return list;
		}
		public ItemForStudyViewModel? ReadElement(ItemForStudySearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. Id: {Id}", model.Id);
			var element = _itemforstudyStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement found. Id: {Id}", element.Id);
			return element;
		}
		public bool Create(ItemForStudyBindingModel model)
		{
			CheckModel(model);
			if (_itemforstudyStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(ItemForStudyBindingModel model)
		{
			CheckModel(model);
			if (_itemforstudyStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(ItemForStudyBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_itemforstudyStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(ItemForStudyBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			_logger.LogInformation("ItemForStudy. Id: {Id}", model.Id);
		}
	}
}
