﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using SchoolContracts.BindingModels;
using SchoolContracts.Models;
using SchoolContracts.ViewModels;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace SchoolDatabase.Models
{
    public class Item : IItemModel
	{
		public int Id { get; private set; }
		[Required]
		public string Name { get; private set; } = string.Empty;
		[Required, Column(TypeName = "decimal (10,2)")]
		public decimal Price { get; private set; }
		[Required]
		public int WorkerId { get; private set; }

		// Затраты на обучение
		[ForeignKey("ItemId")]
		public virtual List<ItemForStudy> ItemForStudy { get; set; } = new();
		public virtual Worker Worker { get; set; } = new();
		public static Item? Create(SchoolDbContext context, ItemBindingModel? model)
		{
			if (model == null)
			{
				return null;
			}
			return new()
			{
				Name = model.Name,
				Price = model.Price,
				Worker = context.Workers.First(x => x.Id == model.WorkerId)
			};
		}
		public void Update(SchoolDbContext context, ItemBindingModel? model)
		{
			if (model == null)
			{
				return;
			}
			Name = model.Name;
			Price = model.Price;
			Worker = context.Workers.First(x => x.Id == model.WorkerId);
		}

		public ItemViewModel GetViewModel => new()
		{
			Id = Id,
			Name = Name,
			Price = Price,
			WorkerId = WorkerId
			//,WorkerFIO = Worker.FIO
		};
	}
}

